package molpage.com;

public final class ViewAliquotManager {
  private UserHttpSess parent;
  private int total;
  private int first;
  private int step;
  private Long sampleId;
  private Long personId;
  private int aliquotCount;

  public ViewAliquotManager(UserHttpSess tmpHttpSessObj) {
    parent = tmpHttpSessObj;
    step = 0;
    total = -1;
    first = 0;
    sampleId = null;
    personId = null;
  }

  public void clearValues() {
    total = -1;
    first = 0;
    step = parent.getViewLimit();
    sampleId = null;
    personId = null;
  }

  public int getAliquotTotal() throws Exception {
    if ((step > 0) && (total < 0))
      total = parent.getAllAliquotsCount( personId, sampleId);
    return total;
  }

  public void changeAliquotTotal( int diff) {
    total += diff;
  }

  public void setAliquotCount( int val) {
    aliquotCount = val;
    if (total < aliquotCount) total = aliquotCount;
  }

  public boolean hasAliquotPrevious() {
    return (step > 0) && (first > 0);
  }

  public void setAliquotPrevious() {
    first -= step;
    if (first < 0) first = 0;
  }

  public boolean hasAliquotNext() {
    return (step > 0) && (first + step < total - 1);
  }

  public void setAliquotNext() {
    first += step;
    if (first >= total) setAliquotPrevious();
  }

  public void setAliquotFirst() {
    first = 0;
  }

  public void setAliquotLast() {
    first = total - step;
    if (first < 0) first = 0;
  }

  public int getAliquotStep() {
    return step;
  }

  public int getAliquotCurrent() {
    return first;
  }

  public void setPersonAndSampleIdDown(Long newPersonId, Long newSampleId) {
    boolean change = false;
    if (newPersonId == null) {
      if (personId != null) change = true;
    }
    else
    if (!newPersonId.equals(personId)) change = true;
    else
    if (newSampleId == null) {
      if (sampleId != null) change = true;
    }
    else
    if (!newSampleId.equals(sampleId)) change = true;
    if (change){
      total = -1;
      first = 0;
      personId = newPersonId;
      sampleId = newSampleId;
    }
  }

  public void setPersonAndSampleIdUp(Long newPersonId, Long newSampleId) {
    boolean change = false;
    if (newPersonId == null) {
      if (personId != null) change = true;
    }
    else
    if (!newPersonId.equals(personId)) change = true;
    else
    if (newSampleId == null) {
      if (sampleId != null) change = true;
    }
    else
    if (!newSampleId.equals(sampleId)) change = true;
    if (change){
      total = -1;
      first = 0;
      personId = newPersonId;
      sampleId = newSampleId;
      ViewSampleManager vsm = parent.getViewSampleManager();
      vsm.setPersonIdUp( personId);
    }
  }

  public Long getSampleId() {
    return sampleId;
  }

  public Long getPersonId() {
    return personId;
  }

  public String getAliquotControlString(boolean down) {
    StringBuffer xyz = new StringBuffer();
    xyz.append("&nbsp;");
    if (total>0){
      if (down) xyz.append(
  "<a href='javascript:jumpFocus(\"bottomLine\")'><img border='0' src='./images/down_d_10_b.gif' title='bottom of page' alt=''></a>"
      ); else xyz.append(	
  "<a href='javascript:scrollTo(0,0)'><img border='0' src='./images/up_d_10_b.gif' title='top of page' alt=''></a>"
      );xyz.append("&nbsp;");
    }
    if ((step > 0) && (total > step)) {
      if (first > 0){
        xyz.append("<button type='button' onclick='gotoFirst()' title='first' class='lines'>")
            .append("<img border='0' src='./images/left_d_10_b.gif' title='first' alt=''>")
            .append("</button>");
        xyz.append("<button type='button' onclick='gotoPrevious()' title='previous' class='lines'>")
            .append("<img border='0' src='./images/left_10_b.gif' title='previous' alt=''>")
            .append("</button>");
      }
     else{
       xyz.append("<button type='button' disabled class='lines'>")
           .append("<img border='0' src='./images/left_d_10_h.gif' alt=''>")
           .append("</button>");
       xyz.append("<button type='button' disabled class='lines'>")
           .append("<img border='0' src='./images/left_10_h.gif' alt=''>")
           .append("</button>");
     }
      xyz.append(first+1)
         .append('-')
         .append(first+aliquotCount)
         .append("&nbsp;of&nbsp;")
         .append(total)
         .append("&nbsp;items&nbsp;");
       if (first + aliquotCount < total){
         xyz.append("<button type='button' onclick='gotoNext()' title='next' class='lines'>")
             .append("<img border='0' src='./images/right_10_b.gif' title='next' alt=''>")
             .append("</button>");
         xyz.append("<button type='button' onclick='gotoLast()' title='last' class='lines'>")
             .append("<img border='0' src='./images/right_d_10_b.gif' title='last' alt=''>")
             .append("</button>");
       }
      else{
        xyz.append("<button type='button' disabled class='lines'>")
            .append("<img border='0' src='./images/right_10_h.gif' alt=''>")
            .append("</button>");
        xyz.append("<button type='button' disabled class='lines'>")
            .append("<img border='0' src='./images/right_d_10_h.gif' alt=''>")
            .append("</button>");
      }
    }
    else
      xyz.append(total)
         .append("&nbsp;items&nbsp;");
    return xyz.toString();
  }

}
