package molpage.com;


import java.util.Stack;
import java.io.IOException;
import javax.servlet.jsp.JspWriter;

import java.util.List;
import java.util.Iterator;
import sims.hibernate.*;
/**
 *
 * <p>Description:Exports metadata & vocabularies in XML-format.
 *     Klasi lieto ExportTechnoology.jsp, ko izsauc no AdminPage.jsp </p>
 *
 * <p>Copyright: Copyright (c) 2008</p>
 *
 */
public final class XMLExport {
  private UserHttpSess session;
  private Stack<String> tagStack;
  private boolean isStarted = false;
  private JspWriter jspOut;
  private String prevName;

  public XMLExport( UserHttpSess tmpHttpSessObj, JspWriter out) {
    session = tmpHttpSessObj;
    jspOut = out;
    tagStack = new Stack<String>();
  }

  private void startTag( String name) throws IOException {
    if (isStarted){
      jspOut.print('>');
      tagStack.push(prevName);
    }
    jspOut.println();
    jspOut.print('<');
    jspOut.print(name);
    prevName = name;
    isStarted = true;
  }

  private void endTag() throws IOException {
    if (isStarted){
      jspOut.print("/>");
      isStarted = false;
    } else {
      jspOut.println();
      prevName = (String)tagStack.pop();
      jspOut.print("</");
      jspOut.print(prevName);
      jspOut.print('>');
    }
  }

  private void attribute(String name, String value) throws IOException {
    jspOut.print(' ');
    jspOut.print(name);
    jspOut.print("=\"");
    jspOut.print(Util.getHTMLString(value));
    jspOut.print('\"');
  }

  private void exportMetaDatums( List plist) throws Exception {
    Iterator piter = plist.iterator();
    while (piter.hasNext()) {
      MetaData obj = (MetaData) piter.next();
      startTag(XMLTags.XMetaDatum);
      attribute(XMLTags.Xname, obj.getVisibleName());
      attribute(XMLTags.Xshortname, obj.getShortName());
      attribute(XMLTags.Xlongname, obj.getLongName());
      attribute(XMLTags.Xuploadname, obj.getUploadNameCom());
      attribute(XMLTags.Xcol_numb, obj.getViewColumnNumber().toString());
      attribute(XMLTags.Xsortable, obj.isSortable() ? "1" : "0");
      attribute(XMLTags.Xeditable, obj.isEditable() ? "1" : "0");
      attribute(XMLTags.Xrequired, obj.isRequired() ? "1" : "0");
      attribute(XMLTags.Xfilter, obj.getPositionInFilterCom().toString());
      attribute(XMLTags.Xcomment, obj.getComment());
      endTag(); //MetaDatum
    }
  }
  public void exportMetaData() throws Exception {
    startTag(XMLTags.XSIMS_Export);
    attribute(XMLTags.Xversion,"1.17");
    List plist = session.getAllObjects(VersionNumber.class);
    if (plist != null && plist.size() > 0) {
      VersionNumber obj = (VersionNumber) plist.get(0);
      attribute(XMLTags.Xversion_numb,obj.getVersionNumber().toString());
      attribute(XMLTags.Xconfiguration,obj.getConfiguration());
    }
    startTag(XMLTags.XTopMetaData);
    plist = session.getAllObjects(TopMetaData.class);
    if (plist != null) exportMetaDatums( plist);
    endTag(); //TopMetaData
    startTag(XMLTags.XMiddleMetaData);
    plist = session.getAllObjects(MiddleMetaData.class);
    if (plist != null) exportMetaDatums( plist);
    endTag(); //MiddleMetaData
    startTag(XMLTags.XBottomMetaData);
    plist = session.getAllObjects(BottomMetaData.class);
    if (plist != null) exportMetaDatums( plist);
    endTag(); //BottomMetaData

    startTag(XMLTags.XVocabularies);
    for (int i = 1; i <= VocabulName.getVocabularyCount() ; i++) {
      startTag(XMLTags.XVocabulary);
      attribute(XMLTags.Xtable, VocabulName.getVocabularyName(i));
      attribute(XMLTags.Xname, VocabulName.getClassNameVT(i));
      Class cvtxy = Class.forName(VocabulName.getClassNameVT(i));
      plist = session.getAllObjects( cvtxy);
      if (plist != null) {
        Iterator iter = plist.iterator();
        while (iter.hasNext()) {
          Persistent obj = (Persistent) iter.next();
          startTag(XMLTags.XItem);
          attribute(XMLTags.Xname, obj.getVisibleName());
          attribute(XMLTags.Xcomment, obj.getComment());
          endTag();//item
        }
      }
      endTag();//vocabulary
    }
    endTag();//vocabularies

    startTag(XMLTags.XIdentificies);
    for (int i = 1; i <= VocabulName.getIdentificyCount() ; i++) {
      startTag(XMLTags.XIdentificy);
      attribute(XMLTags.Xtable, VocabulName.getIdentificyName(i));
      attribute(XMLTags.Xname, VocabulName.getIdentificyClassName(i));
      Class cvtxy = Class.forName(VocabulName.getIdentificyClassName(i));
      plist = session.getAllObjects( cvtxy);
      if (plist != null) {
        Iterator iter = plist.iterator();
        while (iter.hasNext()) {
          Identificy obj = (Identificy) iter.next();
          startTag(XMLTags.XItem);
          attribute(XMLTags.Xname, obj.getVisibleName());
          attribute(XMLTags.Xshortname, obj.getAbbreviation());
          attribute(XMLTags.Xcomment, obj.getComment());
          endTag();//item
        }
      }
      endTag();//Identificy
    }
    endTag();//Identificies

    startTag(XMLTags.XDependencies);
    for (int i = 1; i <= VocabulName.getDependencyCount() ; i++) {
      startTag(XMLTags.XDependency);
      attribute(XMLTags.Xtable, VocabulName.getDependencyName(i));
      attribute(XMLTags.Xname, VocabulName.getDependencyClassName(i));
      attribute(XMLTags.Xindependent, VocabulName.getServerClassName(i));
      attribute(XMLTags.Xdependent, VocabulName.getClientClassName(i));
      Class cvtxy = Class.forName(VocabulName.getDependencyClassName(i));
      plist = session.getAllObjects( cvtxy);
      if (plist != null) {
        Iterator iter = plist.iterator();
        while (iter.hasNext()) {
          Dependency obj = (Dependency) iter.next();
          startTag(XMLTags.XItem);
          attribute(XMLTags.Xindependent, obj.getServer().getVisibleName());
          attribute(XMLTags.Xdependent, obj.getClient().getVisibleName());
          attribute(XMLTags.Xcomment, obj.getComment());
          endTag();//item
        }
      }
      endTag();//Identificy
    }
    endTag();//Identificies

    startTag(XMLTags.XRelationTypes);
    plist = session.getAllObjects( RelationType.class);
    if (plist != null) {
      Iterator iter = plist.iterator();
      while (iter.hasNext()) {
        RelationType obj = (RelationType) iter.next();
        startTag(XMLTags.XItem);
        attribute(XMLTags.Xname, obj.getVisibleName());
        attribute(XMLTags.Xreverse, obj.getRev_relation());
        attribute(XMLTags.Xcomment, obj.getComment());
        endTag();//item
      }
    }
    endTag();//Relations
    startTag(XMLTags.XDataFileTypes);
    plist = session.getAllObjects( DataType.class);
    if (plist != null) {
      Iterator piter = plist.iterator();
      while (piter.hasNext()) {
        DataType obj = (DataType) piter.next();
        startTag(XMLTags.XFileType);
        attribute(XMLTags.Xtype, obj.getVisibleName());
        attribute(XMLTags.Xshortname, obj.getAbbreviation());
        attribute(XMLTags.Xcomment, obj.getComment());
        endTag();
      }
    }
    endTag(); //data file types
    startTag(XMLTags.XProtocolFileTypes);
    plist = session.getAllObjects( ProtocolType.class);
    if (plist != null) {
      Iterator piter = plist.iterator();
      while (piter.hasNext()) {
        ProtocolType obj = (ProtocolType) piter.next();
        startTag(XMLTags.XFileType);
        attribute(XMLTags.Xtype, obj.getVisibleName());
        attribute(XMLTags.Xcomment, obj.getComment());
        endTag();
      }
    }
    endTag(); //protocol file types

    endTag();//SIMS_Export
  }
}
