package molpage.com;
import java.io.File;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import sims.hibernate.*;

import java.util.List;
import java.util.ArrayList;
/**
 *
 *
 * <p>Description:Imports metadata & vocabularies in XML-format.
 *     Klasi lieto ImportTechnoologyUpload.jsp, ko izsauc no AdminPage.jsp </p>
 *
 * <p>Copyright: Copyright (c) 2008</p>
 *
 */
public class XMLImport {
  private UserHttpSess session;
  private String namex = "xyz";
  private List<String> errorlist = null;
  public XMLImport( UserHttpSess tmpHttpSessObj) {
    session = tmpHttpSessObj;
    errorlist = new ArrayList<String>();
  }
  
  private int processDataFileTypes( Element root) throws Exception {
    int result = 0;
    NodeList nodeLst1 = root.getElementsByTagName(XMLTags.XFileType);
    for (int i = 0; i < nodeLst1.getLength(); i++){
      Element item = (Element) nodeLst1.item(i);
      String type = item.getAttribute(XMLTags.Xtype);
      if (type != null && !type.trim().equals("")) {
        if (session.checkObjectByName(DataType.class, type) == null) {
          String comment = item.getAttribute(XMLTags.Xcomment);
          DataType obj = DataType.class.newInstance();
          String abbreviation = item.getAttribute(XMLTags.Xshortname);
          obj.setAbbreviation( abbreviation);
          if (!session.addPersistentWithoutClose(obj, type, comment))
              result = 1;
        }
      }
    }
	return result;
  }
  private int processProtocolFileTypes( Element root) throws Exception {
    int result = 0;
    NodeList nodeLst1 = root.getElementsByTagName(XMLTags.XFileType);
    for (int i = 0; i < nodeLst1.getLength(); i++){
      Element item = (Element) nodeLst1.item(i);
      String type = item.getAttribute(XMLTags.Xtype);
      if (type != null && !type.trim().equals("")) {
        if (session.checkObjectByName(ProtocolType.class, type) == null) {
          String comment = item.getAttribute(XMLTags.Xcomment);
          ProtocolType obj = ProtocolType.class.newInstance();
          if (!session.addPersistentWithoutClose(obj, type, comment))
              result = 1;
        }
      }
    }
	return result;
  }
private int processRelationTypes( Element root) throws Exception {
  int result = 0;
  NodeList nodeLst2 = root.getElementsByTagName(XMLTags.XItem);
  for (int j = 0; j < nodeLst2.getLength(); j++) {
    Element item = (Element) nodeLst2.item(j);
    String name = item.getAttribute(XMLTags.Xname);
    if (name != null && !name.trim().equals("")) {
      if (session.checkObjectByName(RelationType.class, name) == null) {
        RelationType obj = (RelationType) RelationType.class.newInstance();
        String reverse = item.getAttribute(XMLTags.Xreverse);
        obj.setRev_relation(reverse);
        String comment = item.getAttribute(XMLTags.Xcomment);
        if (!session.addPersistentWithoutClose(obj, name, comment))
          result = 1;
      }
    }
  }
  return result;
}

private int processDependencies( Element root) throws Exception {
  int result = 0;
  NodeList nodeLst1 = root.getElementsByTagName(XMLTags.XDependency);
  for (int i = 0; i < nodeLst1.getLength(); i++){
    Element dependency = (Element) nodeLst1.item(i);
    String clasname = dependency.getAttribute(XMLTags.Xname);
    Class cvtxy = Class.forName(clasname);
    clasname = dependency.getAttribute(XMLTags.Xindependent);
    Class cvtserver = Class.forName(clasname);
    clasname = dependency.getAttribute(XMLTags.Xdependent);
    Class cvtclient = Class.forName(clasname);
    NodeList nodeLst2 = dependency.getElementsByTagName(XMLTags.XItem);
    for (int j = 0; j < nodeLst2.getLength(); j++) {
      Element item = (Element) nodeLst2.item(j);
      String serverName = item.getAttribute(XMLTags.Xindependent);
      Persistent server = session.getObjectByName(cvtserver, serverName);
      String clientName = item.getAttribute(XMLTags.Xdependent);
      Persistent client = session.getObjectByName(cvtclient, clientName);
      if (server == null || client == null) {
        errorlist.add("Invalid dependency");
        continue;
      }
      if (session.checkDependencyByIds(cvtxy, server.getId(), client.getId()) == null) {
        String comment = item.getAttribute(XMLTags.Xcomment);
        Dependency obj = (Dependency) cvtxy.newInstance();
        obj.setClient(client);
        obj.setServer(server);
        if (!session.addPersistentWithoutClose(obj, "", comment))
          result = 1;
      }
    }
  }
  return result;
}

private int processIdentificies( Element root) throws Exception {
  int result = 0;
  NodeList nodeLst1 = root.getElementsByTagName(XMLTags.XIdentificy);
  for (int i = 0; i < nodeLst1.getLength(); i++){
    Element identificy = (Element) nodeLst1.item(i);
    String clasname = identificy.getAttribute(XMLTags.Xname);
    Class cvtxy = Class.forName(clasname);
    NodeList nodeLst2 = identificy.getElementsByTagName(XMLTags.XItem);
    for (int j = 0; j < nodeLst2.getLength(); j++) {
      Element item = (Element) nodeLst2.item(j);
      String name = item.getAttribute(XMLTags.Xname);
      if (name != null && !name.trim().equals("")) {
        if (session.checkObjectByName(cvtxy, name) == null) {
          Identificy obj = (Identificy) cvtxy.newInstance();
          String abbreviation = item.getAttribute(XMLTags.Xshortname);
          obj.setAbbreviation( abbreviation);
          String comment = item.getAttribute(XMLTags.Xcomment);
          if (!session.addPersistentWithoutClose(obj, name, comment))
            result = 1;
        }
      }
    }
  }
  return result;
}

  private int processVocabularies( Element root) throws Exception {
    int result = 0;
    NodeList nodeLst1 = root.getElementsByTagName(XMLTags.XVocabulary);
    for (int i = 0; i < nodeLst1.getLength(); i++){
      Element vocabulary = (Element) nodeLst1.item(i);
      String clasname = vocabulary.getAttribute(XMLTags.Xname);
      Class cvtxy = Class.forName(clasname);

      NodeList nodeLst2 = vocabulary.getElementsByTagName(XMLTags.XItem);
      for (int j = 0; j < nodeLst2.getLength(); j++) {
        Element item = (Element) nodeLst2.item(j);
        String name = item.getAttribute(XMLTags.Xname);
        if (name != null && !name.trim().equals("")) {
          if (session.checkObjectByName(cvtxy, name) == null) {
            String comment = item.getAttribute(XMLTags.Xcomment);
            Persistent obj = (Persistent) cvtxy.newInstance();
            if (!session.addPersistentWithoutClose(obj, name, comment))
              result = 1;
          }
        }
      }
    }
    return result;
  }

  private int processMetaData( Element root, Class metaClass) throws Exception {
    int result = 0;
    NodeList nodeLst1 = root.getElementsByTagName(XMLTags.XMetaDatum);
    for (int i = 0; i < nodeLst1.getLength(); i++){
      Element metaDatum = (Element) nodeLst1.item(i);
      String name = metaDatum.getAttribute(XMLTags.Xname);
      if (name != null && !name.trim().equals("")) {
        if (session.checkObjectByName( metaClass, name) == null) {
          MetaData obj = (MetaData) metaClass.newInstance();
          String value = metaDatum.getAttribute(XMLTags.Xshortname);
          obj.setShortName(value);
          value = metaDatum.getAttribute(XMLTags.Xlongname);
          obj.setLongName(value);
          value = metaDatum.getAttribute(XMLTags.Xuploadname);
          obj.setUploadName(value);
          value = metaDatum.getAttribute(XMLTags.Xcol_numb);
          obj.setViewColumnNumber(Util.getInteger0( value));
          value = metaDatum.getAttribute(XMLTags.Xsortable);
          obj.setSortableX(Util.getInteger0( value));
          value = metaDatum.getAttribute(XMLTags.Xeditable);
          obj.setEditableX(Util.getInteger0( value));
          value = metaDatum.getAttribute(XMLTags.Xrequired);
          if (value!=null) //may be null for old versions (6 aug 2009)
            obj.setIsRequiredX(Util.getInteger0( value));
          value = metaDatum.getAttribute(XMLTags.Xfilter);
          obj.setPositionInFilter(Util.getInteger0( value));
          String comment = metaDatum.getAttribute(XMLTags.Xcomment);
          if (!session.addPersistentWithoutClose(obj, name, comment)) result = 1;
        }
      }
    }
    return result;
  }

  private int processRoot( Element root) throws Exception{
    int result = 0;
    String version = root.getAttribute(XMLTags.Xversion);
    if (version==null||!version.equals("1.17")){
      errorlist.add("Invalid version");
      return 1;
    }
    String versnumb = root.getAttribute(XMLTags.Xversion_numb);
    String config = root.getAttribute(XMLTags.Xconfiguration);
    version = session.checkDataBaseVersion( versnumb, config);
    if (!version.equals("OK")){
      errorlist.add( version);
      return 1;
    }
    NodeList nodeLst1 = root.getElementsByTagName(XMLTags.XTopMetaData);
    int num = nodeLst1.getLength();
    if ( num > 1){
      errorlist.add("Invalid top meta data");
      return 1;
    }
    if (num == 1) result = processMetaData( (Element)nodeLst1.item(0),
                                            TopMetaData.class);
    if (result > 0) return result;
    nodeLst1 = root.getElementsByTagName(XMLTags.XMiddleMetaData);
    num = nodeLst1.getLength();
    if ( num > 1){
      errorlist.add("Invalid middle meta data");
      return 1;
    }
    if (num == 1) result = processMetaData( (Element)nodeLst1.item(0),
                                            MiddleMetaData.class);
    if (result > 0) return result;
    nodeLst1 = root.getElementsByTagName(XMLTags.XBottomMetaData);
    num = nodeLst1.getLength();
    if ( num > 1){
      errorlist.add("Invalid bottom meta data");
      return 1;
    }
    if (num == 1) result = processMetaData( (Element)nodeLst1.item(0),
                                            BottomMetaData.class);
    if (result > 0) return result;

    nodeLst1 = root.getElementsByTagName(XMLTags.XVocabularies);
    num = nodeLst1.getLength();
    if ( num > 1){
      errorlist.add("Invalid vocabularies");
      return 1;
    }
    if (num == 1)  result = processVocabularies( (Element)nodeLst1.item(0));
    if (result > 0) return result;
    nodeLst1 = root.getElementsByTagName(XMLTags.XIdentificies);
    num = nodeLst1.getLength();
    if ( num > 1){
      errorlist.add("Invalid identificies");
      return 1;
    }
    if (num == 1)  result = processIdentificies( (Element)nodeLst1.item(0));
    if (result > 0) return result;
    nodeLst1 = root.getElementsByTagName(XMLTags.XDependencies);
    num = nodeLst1.getLength();
    if ( num > 1){
      errorlist.add("Invalid dependencies");
      return 1;
    }
    if (num == 1)  result = processDependencies( (Element)nodeLst1.item(0));
    if (result > 0) return result;
    nodeLst1 = root.getElementsByTagName(XMLTags.XRelationTypes);
    num = nodeLst1.getLength();
    if ( num > 1){
      errorlist.add("Invalid relation types");
      return 1;
    }
    if (num == 1)  result = processRelationTypes( (Element)nodeLst1.item(0));
//  import Protocol File Types
    nodeLst1 = root.getElementsByTagName(XMLTags.XProtocolFileTypes);
    num = nodeLst1.getLength();
    if ( num > 1){
      errorlist.add("Invalid protocol file type");
      return 1;
    }
    if (num == 1) result = processProtocolFileTypes( (Element)nodeLst1.item(0));
    if (result > 0) return result;
//import Data File Types
    nodeLst1 = root.getElementsByTagName(XMLTags.XDataFileTypes);
    num = nodeLst1.getLength();
    if ( num > 1){
      errorlist.add("Invalid data file type");
      return 1;
    }
    if (num == 1) result = processDataFileTypes( (Element)nodeLst1.item(0));
    
    return result;
  }
  public int importTechn() throws Exception {
    int result = 0;
    try {
      File file = session.getZipTempFile();
      DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
      DocumentBuilder db = dbf.newDocumentBuilder();
      Document doc = db.parse(file);
      Element root = doc.getDocumentElement();
      root.normalize();
      result = processRoot( root);
    }
    catch (Exception e) {
      if (e!=null){
      java.io.CharArrayWriter cw = new java.io.CharArrayWriter();
      java.io.PrintWriter pw = new java.io.PrintWriter(cw,true);
      e.printStackTrace(pw);
      Util.writeLog(cw.toString(),"","");
      errorlist.add(e.getMessage());
     }
     result = 1;
    }
    if (result > 0)
      errorlist.add("import partially incomplete");
    return result;
  }

  public int getErrorCount() {
    return errorlist.size();
  }
  public String getErrorMsg(int index) {
    return (String)errorlist.get(index);
  }
  public String getTechnologyName() {
    return namex;
  }
}


