package sims.hibernate;

import java.util.HashMap;
import java.util.List;
import java.util.Iterator;


public final class FileString {
  private String oldContents;
  private String newContents;
  private String userFileName;
  private String userFilePath;
  private String userTypeId;
  private String kindOfFile;
  private String newSuffix = null;
  private HashMap<Long,String> typeBook = null;
  private boolean noContent = false;
  private int level;
  private boolean isData;
  private Long sysId;

//structure of filestring
// \rsuffix\rfilename\rtypid\r.....

  public FileString( int level, boolean isData, String cont, Long sysId){
    oldContents = cont;
    newContents = cont;
    userFileName = "";
    userFilePath = "";
    userTypeId = "0";
    kindOfFile = "";
    this.level = level;
    this.isData = isData;
    this.sysId = sysId;
  }

  public void clearFileString(){
    oldContents = "";
    newContents = "";
    userFileName = "";
    userFilePath = "";
    userTypeId = "0";
    kindOfFile = "";
    noContent = true;
  }

  public Long getUserTypeId(){
    return new Long(userTypeId);
  }

  public String getUserFilePath(){
    return userFilePath;
  }

  public String getKindOfFile(){
    return kindOfFile;
  }

  public void setTypeBook( List types){
    typeBook = new HashMap<Long,String>();
    typeBook.put(new Long(0),"");
    if (types!=null){
      Iterator typesIter = types.iterator();
      while (typesIter.hasNext()) {
        Persistent type = (Persistent) typesIter.next();
        typeBook.put(type.getId(),type.getVisibleName());
      }
    }
  }

  private String getFreeSuffix(){
    int i = 1, j, k = 0, l;
    if (oldContents==null || oldContents.length() < 5 ||
        oldContents.charAt(0)!='\r') return "1";
    while (i < oldContents.length()){
      j = oldContents.indexOf('\r', i);
      if (j < 0) break;
      String suffix = oldContents.substring(i,j);
      l = MyUtil.getIntegerValue(suffix);
      if (l > k) k = l;
      i = j + 1;
      j = oldContents.indexOf('\r', i);
      if (j < 0) break;
      i = j + 1;
      j = oldContents.indexOf('\r', i);
      if (j < 0) break;
      i = j + 1;
    }
    return String.valueOf( k + 1);
  }

// used by delete all
  public String[] getAllFileNames(){
//    String prefix = getPrefix( isAssay, isData);
    int i = 1, j, k = 0;
    if (oldContents==null || oldContents.length() < 5 ||
        oldContents.charAt(0)!='\r') return null;
//1)get file number
    while (i < oldContents.length()){
      j = oldContents.indexOf('\r', i);
      if (j < 0) break;
      i = j + 1;
      j = oldContents.indexOf('\r', i);
      if (j < 0) break;
      i = j + 1;
      j = oldContents.indexOf('\r', i);
      if (j < 0) break;
      i = j + 1;
      k++;
    }
    String[] names = new String[k];
    k = 0; i = 1;
//2)get file names
    while (i < oldContents.length()){
      j = oldContents.indexOf('\r', i);
      if (j < 0) break;
      String suffix = oldContents.substring(i,j);
      i = j + 1;
      j = oldContents.indexOf('\r', i);
      if (j < 0) break;
      names[k] = getFileName(suffix);
      i = j + 1;
      j = oldContents.indexOf('\r', i);
      if (j < 0) break;
      i = j + 1;
      k++;
    }
    return names;
  }

  // used by delete deleted
    public String[] getDeletedFileNames(){
//      String prefix = getPrefix( isAssay, isData);
      int iOld, iNew, j, k, kOld = 0, kNew = 0;
      String newSuf;
      if (oldContents==null || oldContents.length() < 5 ||
          oldContents.charAt(0)!='\r') return null;
      iOld = 1;
      while (iOld < oldContents.length()){
        j = oldContents.indexOf('\r', iOld);
        if (j < 0) break;
        iOld = j + 1;
        j = oldContents.indexOf('\r', iOld);
        if (j < 0) break;
        iOld = j + 1;
        j = oldContents.indexOf('\r', iOld);
        if (j < 0) break;
        iOld = j + 1;
        kOld++;
      }
      iNew = 1; newSuf = null;
      while (iNew < newContents.length()){
        j = newContents.indexOf('\r', iNew);
        iNew = j + 1;
        j = newContents.indexOf('\r', iNew);
        iNew = j + 1;
        j = newContents.indexOf('\r', iNew);
        iNew = j + 1;
        kNew++;
      }
      if (kOld <= kNew) return null;
      String[] names = new String[kOld - kNew];
      iNew = 1;
      if (iNew < newContents.length()){
        j = newContents.indexOf('\r', iNew);
        newSuf = newContents.substring(iNew,j);
        iNew = j + 1;
        j = newContents.indexOf('\r', iNew);
        iNew = j + 1;
        j = newContents.indexOf('\r', iNew);
        iNew = j + 1;
      } else newSuf = null;
      k = 0; iOld = 1;
      while (iOld < oldContents.length()){
        j = oldContents.indexOf('\r', iOld);
        String suffix = oldContents.substring(iOld,j);
        iOld = j + 1;
        if (!suffix.equals(newSuf)){
          names[k] = getFileName(suffix);
//          names[k] = prefix+"_"+sysId.toString()+"_"+suffix;
          k++;
        }
        else {
          if (iNew < newContents.length()) {
            j = newContents.indexOf('\r', iNew);
            newSuf = newContents.substring(iNew, j);
            iNew = j + 1;
            j = newContents.indexOf('\r', iNew);
            iNew = j + 1;
            j = newContents.indexOf('\r', iNew);
            iNew = j + 1;
          }
          else newSuf = null;
        }
        j = oldContents.indexOf('\r', iOld);
        iOld = j + 1;
        j = oldContents.indexOf('\r', iOld);
        iOld = j + 1;
      }
      return names;
    }

  public String getHTML( Top topObj){
    User owner = null;
    UserGroup ownerGroup = null;
    if (topObj!=null){
      owner = topObj.getOwner();
      ownerGroup = topObj.getOwnerGroup();
    }
    String ownerStr1 = (owner==null)?"0":owner.getId().toString();
    String ownerStr2 = (ownerGroup==null)?"0":ownerGroup.getId().toString();
    String prefix = getPrefix( level, isData);
    StringBuffer xyz = new StringBuffer();
    int i = 1; int j, k = 0;
    if (newContents!=null && newContents.length() > 0 && newContents.charAt(0)=='\r')
    while (i < newContents.length()){
      j = newContents.indexOf('\r', i);
      if (j < 0) break;
      String suffix = newContents.substring(i,j);
      i = j + 1;
      j = newContents.indexOf('\r', i);
      if (j < 0) break;
      String fileName = newContents.substring(i,j);
      i = j + 1;
      j = newContents.indexOf('\r', i);
      if (j < 0) break;
      String typeId = newContents.substring(i,j);

      String key = prefix + String.valueOf(++k);
      xyz.append("<span id =\"s")
         .append(key)
         .append("\">");
//visible part - file reference
      xyz.append("<a href=\"javascript:checkFB('")
         .append(getFileName(suffix))
         .append("','")
         .append(fileName)
         .append("','")
         .append(ownerStr1)
         .append("','")
         .append(ownerStr2)
         .append("');\">");
      xyz.append(fileName)
         .append("</a>");
//visible part - file type
      xyz.append("&nbsp; of type: &nbsp;")
         .append((String)typeBook.get( MyUtil.getLong( typeId)))
         .append("&nbsp;");
//visible part - delete button
     xyz.append("<input type=\"submit\" value=\"Remove\" title=\"remove\"")
        .append(" onclick='return deleteFile(\"s")
        .append(key)
        .append("\")' style=\"height:20px;\">");
//unvisible part - user file name
      xyz.append("<input type=\"hidden\" name =\"")
         .append(key)
         .append("\" value=\"")
         .append(fileName)
         .append("\">");
//unvisible part - file name suffix
      xyz.append("<input type=\"hidden\" name =\"h")
         .append(key)
         .append("\" value=\"")
         .append(suffix)
         .append("\">");
//unvisible part - file type id
     xyz.append("<input type=\"hidden\" name =\"t")
        .append(key)
        .append("\" value=\"")
        .append(typeId)
        .append("\"><br>");
      xyz.append("</span>\r\n");
      i = j + 1;
    }
//unvisible part - number of files   "lprefix"
    xyz.append("<input type=\"hidden\" name =\"l")
       .append(prefix)
       .append("\" value=\"")
       .append(String.valueOf(k))
       .append("\"/>");
//unvisible part - local or ftp file? "fprefix"
       xyz.append("<input type=\"hidden\" name =\"f")
          .append(prefix)
          .append("\" id=\"f")
          .append(prefix)
          .append("\" value=\"")    //"u" or "f"
          .append(kindOfFile)
          .append("\"/>");
//unvisible part - full user file name  "vprefix"
       xyz.append("<input type=\"hidden\" name =\"v")
          .append(prefix)
          .append("\" id=\"v")
          .append(prefix)
          .append("\" value=\"")
          .append(userFilePath)
          .append("\"/>");
//visible part - user file name  'prefix"
    xyz.append("<input type=\"text\" size=\"30\" name =\"")
       .append(prefix)
       .append("\" id=\"")
       .append(prefix)
       .append("\" readonly style=\"background-color:#E5EAF0\"")
       .append(" value=\"")
       .append(userFileName)
       .append("\"/>");
    return xyz.toString();
  }

  private String getFileName( String suffix){
    String prefix = getPrefix( level, isData);
    String ident = (sysId!=null)?sysId.toString():"";
    String extension = (isData)?".dat":".pro";
    return prefix+"_"+ident+"_"+suffix+extension;
  }

  public String getFileName( Long sysId){
    String prefix = getPrefix( level, isData);
    String extension = (isData)?".dat":".pro";
    return (userFilePath!=null && !userFilePath.equals(""))?
          (prefix+"_"+sysId.toString()+"_"+newSuffix+extension):null;
  }
//after user dialog creates string to be saved
//and prepares info for methods called by UserHttpSession
//-getKindOfFile
//-getFileName
//-getUserFilePath
/*  
  public String getContents( RequestParser request){
    String prefix = getPrefix( level, isData);
    int k, l;
    if (noContent){ noContent = false; return "";}
    StringBuffer xyz = new StringBuffer();
    String suffix, fileName, typeId;
    String count = request.getParameter("l" + prefix);
    k = 0; l = (count!=null)?MyUtil.getIntegerValue(count):10;
    while (k++ < l){
      String key = prefix + String.valueOf(k);
      suffix = request.getParameter("h" + key);
      if (suffix != null) {
        fileName = request.getParameter(key);
        typeId = request.getParameter("t" + key);
        xyz.append('\r')
           .append(suffix)
           .append('\r')
           .append(fileName)
           .append('\r')
           .append(typeId);
      }
    }
    xyz.append('\r');
    newContents = xyz.toString();

    userFilePath = request.getParameter("v"+prefix);
    userFileName = request.getParameter(prefix);
    userTypeId = request.getParameter("t"+prefix);
    kindOfFile = request.getParameter("f"+prefix);
// Append to contents new file entered
    if (userFilePath!=null && !userFilePath.equals("")){
       newSuffix = getFreeSuffix();
       xyz.append(newSuffix)
          .append('\r')
          .append(userFileName)
          .append('\r')
          .append(userTypeId)
          .append('\r');
    }
    return xyz.toString();
  }
*/  
//------------------------- dynamic methods called from UserHttpSess ---------------------
  public String setContents( StringBuffer xyz, String userFilePath, String userFileName,
		  String userTypeId, String kindOfFile){
    this.newContents = xyz.toString();
	this.userFilePath = userFilePath;
	this.userFileName = userFileName;
	this.userTypeId = userTypeId;
	this.kindOfFile = kindOfFile;
    if (this.userFilePath!=null && !this.userFilePath.equals("")){
        newSuffix = getFreeSuffix();
        xyz.append(newSuffix)
           .append('\r')
           .append(this.userFileName)
           .append('\r')
           .append(this.userTypeId)
           .append('\r');
     }
     return xyz.toString();
  }
  public boolean noContent(){
	boolean result = noContent;  
    if (noContent) noContent = false; 
	return result;  
  }
  public String getPrefix(){
    switch (level){
      case 0: return isData?"topd":"topp";
      case 1: return isData?"middled":"middlep";
      default: return isData?"bottomd":"bottomp";
    }
  }
//------------------------- static methods -------------------------
  private static String getPrefix( int level, boolean isData){
    switch (level){
      case 0: return isData?"topd":"topp";
      case 1: return isData?"middled":"middlep";
      default: return isData?"bottomd":"bottomp";
    }
  }

  public static String getNewFileName( int level, boolean isData, Long sysId){
      String prefix = getPrefix( level, isData);
      String extention = (isData)?".dat":".pro";
      return prefix + '_' + sysId.toString() + "_1" + extention;
  }

  public static String getNewFileName( int level, boolean isData,
                                       Long sysId, int suffix){
      String prefix = getPrefix( level, isData);
      String extention = (isData)?".dat":".pro";
      return prefix + '_' + sysId.toString() + '_' +
                                String.valueOf(suffix) + extention;
  }

  public static String getNewFileName( int level, boolean isData,
                                       Long sysId, String suffix){
      String prefix = getPrefix( level, isData);
      String extention = (isData)?".dat":".pro";
      return prefix + '_' + sysId.toString() + '_' +
                                suffix + extention;
  }

  public static String getNewContents( String fileName, String typeId){
      return "\r1\r" + fileName + '\r' + typeId + '\r';
  }

  public static String getNewContents( String fileName, String typeId, int suffix){
      return String.valueOf(suffix) + '\r' + fileName + '\r' + typeId + '\r';
  }

  public static String getFileList( int level, boolean isData,
    String source, Long sysId, Top topObj, boolean isPending){
    String num = null;
    if (source!=null && source.length() > 5 && source.charAt(0)=='\r'){
      StringBuffer xyz = new StringBuffer();
      xyz.append("<img border=\"0\" title=\"");
      int i = 1; int k = 0; int j;
      while (i < source.length()) {
        j = source.indexOf('\r', i);
        if (j < 0)break;
        if (k == 0) num = source.substring(i, j);
        i = j + 1;
        j = source.indexOf('\r', i);
        if (j < 0)break;
        if (k++ > 0) xyz.append(',');
        xyz.append(source.substring(i, j));
        i = j + 1;
        j = source.indexOf('\r', i);
        if (j < 0)break;
        i = j + 1;
      }
      if (k > 1){
        xyz.append("\" src=\"./images/");
        xyz.append( (isData) ? "multifiles_a.gif\">" : "multiprotofiles_a.gif\">");
      }
      else
      if (isPending){
        xyz.append("\" src=\"./images/hourglass_a.gif\">");
      }
      else{
        User owner = (topObj==null)?null:topObj.getOwner();
        UserGroup ownerGroup = (topObj==null)?null:topObj.getOwnerGroup();
        String ownerStr1 = (owner==null)?"0":owner.getId().toString();
        String ownerStr2 = (ownerGroup==null)?"0":ownerGroup.getId().toString();
        xyz.append("\" src=\"./images/");
        xyz.append( (isData) ? "datafiles_a.gif" : "protofiles_a.gif");
        xyz.append("\" onmouseover=\"this.src='./images/");
        xyz.append( (isData) ? "datafiles_b.gif'" : "protofiles_b.gif'");
        xyz.append("\" onmouseout=\"this.src='./images/");
        xyz.append( (isData) ? "datafiles_a.gif'" : "protofiles_a.gif'");
        xyz.append("\" onclick=\"checkFB('")
            .append(getPrefix( level, isData))
            .append('_')
            .append(sysId.toString())
            .append('_')
            .append(num)
            .append( (isData) ? ".dat" : ".pro")
            .append("',this.title,'")
            .append(ownerStr1)
            .append("','")
            .append(ownerStr2)
            .append("');\">");   //',!!!!!!!!!!
      }
      return xyz.toString();
    }
    return null;
  }

  public static String getFileList( String source){
    if (source!=null && source.length() > 5 && source.charAt(0)=='\r'){
      StringBuffer xyz = new StringBuffer();
      int i = 1; int k = 0; int j;
      while (i < source.length()) {
        j = source.indexOf('\r', i);
        if (j < 0)break;
        i = j + 1;
        j = source.indexOf('\r', i);
        if (j < 0)break;
        if (k++ > 0) xyz.append(',');
        xyz.append(source.substring(i, j));
        i = j + 1;
        j = source.indexOf('\r', i);
        if (j < 0)break;
        i = j + 1;
      }
      return xyz.toString();
    }
    return null;
  }

  public static int getFileCount( String contents){
	    int i = 1, j, k = 0;
	    if (contents==null || contents.length() < 5 ||
	        contents.charAt(0)!='\r') return 0;
	    while (i < contents.length()){
	      j = contents.indexOf('\r', i);
	      if (j < 0) break;
	      i = j + 1;
	      j = contents.indexOf('\r', i);
	      if (j < 0) break;
	      i = j + 1;
	      j = contents.indexOf('\r', i);
	      if (j < 0) break;
	      i = j + 1;
	      k++;
	    }
	    return k;
  }  

//used by delete study group with assays
  public static String[] getAllFileNames( int level, boolean isData,
      String contents, Long sysId){
    int i = 1, j, k = 0;
    if (contents==null || contents.length() < 5 ||
        contents.charAt(0)!='\r') return null;
//1)get file number
    while (i < contents.length()){
      j = contents.indexOf('\r', i);
      if (j < 0) break;
      i = j + 1;
      j = contents.indexOf('\r', i);
      if (j < 0) break;
      i = j + 1;
      j = contents.indexOf('\r', i);
      if (j < 0) break;
      i = j + 1;
      k++;
    }
    String[] names = new String[k];
    k = 0; i = 1;
//2)get file names
    while (i < contents.length()){
      j = contents.indexOf('\r', i);
      if (j < 0) break;
      String suffix = contents.substring(i,j);
      i = j + 1;
      j = contents.indexOf('\r', i);
      if (j < 0) break;
      names[k] = getNewFileName( level, isData, sysId, suffix);
      i = j + 1;
      j = contents.indexOf('\r', i);
      if (j < 0) break;
      i = j + 1;
      k++;
    }
    return names;
  }

//used by Martins_EGA  getAllFileNamesII( 1, true, contents, sysId);
  public static String[] getAllFileNamesII( int level, boolean isData,
      String contents, Long sysId){
    int i = 1, j, k = 0;
    if (contents==null || contents.length() < 5 || contents.charAt(0)!='\r') return null;
//1)get file number
    while (i < contents.length()){
      j = contents.indexOf('\r', i);
      if (j < 0) break;
      i = j + 1;
      j = contents.indexOf('\r', i);
      if (j < 0) break;
      i = j + 1;
      j = contents.indexOf('\r', i);
      if (j < 0) break;
      i = j + 1;
      k++;
    }
    String[] names = new String[k];
    k = 0; i = 1;
//2)get file names
    String preffix = getPrefix(level, isData); 
    String extention = (isData)?".dat":".pro";
    String ident = sysId.toString();
    while (i < contents.length()){
      j = contents.indexOf('\r', i);
      if (j < 0) break;
      String suffix = contents.substring(i,j);
      i = j + 1;
      j = contents.indexOf('\r', i);
      if (j < 0) break;
      names[k] = preffix + '_' + ident + "_" + suffix + extention;
      i = j + 1;
      j = contents.indexOf('\r', i);
      if (j < 0) break;
      i = j + 1;
      k++;
    }
    return names;
  }
  
  public static String getExportStatusImage( Integer exportStatus){
    String rezult = "";
    if (exportStatus!= null){
      int exportStatusValue = exportStatus.intValue();
      if (exportStatusValue > 0){
    	rezult = "<img border=\"0\" title=\"submit EGA\" src=\"./images/export_\r.gif\">";
        switch (exportStatusValue){
          case 1000: rezult = rezult.replace('\r','p');break;
          case 2000: rezult = rezult.replace('\r','s');break;
          case 3000: rezult = rezult.replace('\r','f');break;
          case 4000: rezult = rezult.replace('\r','f');break;
          case 5000: rezult = rezult.replace('\r','c');break;
        }
      }	
    }
    return rezult;  	
  }  
}
